@@ -29,7 +29,7 @@
 //   You should have received a copy of the GNU General Public License
 //   along with this program; if not, see <http://www.gnu.org/licenses/>.
 //
-//    $Id: xptopen.cpp 12742 2015-01-24 15:09:30Z schloegl $
+//    $Id: xptopen.cpp 12888 2015-12-22 10:28:53Z schloegl $
 //    Copyright (C) 2010,2011,2012,2013 Alois Schloegl <alois.schloegl@ist.ac.at>
 //    This function is part of the NaN-toolbox
 //    http://pub.ist.ac.at/~schloegl/matlab/NaN/
@@ -50,8 +50,8 @@ SPSS file format
 // http://cvs.savannah.gnu.org/pspp/doc/data-file-format.texi?root=pspp&content-type=text%2Fplain
 */
 
-#define TEST_CONVERSION 2  // 0: ieee754, 1: SAS converter (big endian bug), 2: experimental
-#define DEBUG 0
+#define TEST_CONVERSION 0  // 0: ieee754, 1: SAS converter (big endian bug), 2: experimental
+#define DEBUG 1
 
 #include <ctype.h>
 #include <math.h>
@@ -63,6 +63,10 @@ SPSS file format
 #include <time.h>
 #include "mex.h"
 
+#ifndef typeof
+#define typeof __typeof__
+#endif
+
 #ifdef tmwtypes_h
   #if (MX_API_VER<=0x07020000)
     typedef int mwSize;
@@ -236,20 +240,6 @@ double xpt2d(uint64_t x);
 uint64_t d2xpt(double x);
 double tm_time2gdf_time(struct tm *t);
 
-/*
-	compare first n characters of two strings, ignore case
- */
-int strncmpi(const char* str1, const char* str2, size_t n)
-{
-	unsigned int k=0;
-	int r=0;
-	while (!r && str1[k] && str2[k] && (k<n)) {
-		r = tolower(str1[k]) - tolower(str2[k]);
-		k++;
-	}
-	return(r);
-}
-
 void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const mxArray *PInputs[])
 {
 	const char L1[] = "HEADER RECORD*******LIBRARY HEADER RECORD!!!!!!!000000000000000000000000000000  ";
@@ -315,8 +305,8 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 	}
 
 	fid = fopen(fn,Mode);
-	if (fid < 0) {
-	        mexWarnMsgTxt("Warning XPTOPEN: suppor for SPSS file format is very experimantal ( do not use it for production use)\n");
+	if (fid == NULL) {
+	        mexErrMsgTxt("Can not open file!\n");
 		}
 
 	if (Mode[0]=='r' || Mode[0]=='a' ) {
@@ -386,7 +376,7 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 				}
 
 				if (rec_type != 2) 
-					;//error('invalid SPSS file');
+					mexErrMsgTxt("invalid SPSS file\n");
 
 				c2 += fread(VarNames+9*ns,1,8,fid);
 				VarNames[9*ns+8] = 0;
@@ -483,8 +473,8 @@ if present.
 			char typ; int32_t len;
 			char flagSWAP = (((__BYTE_ORDER == __BIG_ENDIAN) && LittleEndian) || ((__BYTE_ORDER == __LITTLE_ENDIAN) && !LittleEndian));
 			do {
-				fread(&typ,1,1,fid);
-				fread(&len,4,1,fid);
+				if (!fread(&typ,1,1,fid)) break;
+				if (!fread(&len,4,1,fid)) break;
 				if (flagSWAP) bswap_32(len);
 				fseek(fid,len,SEEK_CUR);
 			} while (len);
@@ -524,7 +514,7 @@ if present.
 				switch (typlist[k]) {
 				case 0xfb:
 					R[k] = mxCreateDoubleMatrix(M, 1, mxREAL);
-					for (size_t m = 0; m < M; m++) {
+					for (typeof(M) m = 0; m < M; m++) {
 						int8_t d = *(int8_t*)(data+bi[k]+m*bi[NS]);
 						((double*)mxGetData(R[k]))[m] = (d>100) ? NaN : d;
 					}
@@ -535,7 +525,7 @@ if present.
 						int16_t d = (int16_t) bswap_16(*(uint16_t*)(data+bi[k]+m*bi[NS]));
 						((double*)mxGetData(R[k]))[m] = (d>32740) ? NaN : d;
 					}
-					else for (size_t m = 0; m < M; m++) {
+					else for (typeof(M) m = 0; m < M; m++) {
 						int16_t d = *(int16_t*)(data+bi[k]+m*bi[NS]);
 						((double*)mxGetData(R[k]))[m] = (d>32740) ? NaN : d;
 					}
@@ -546,7 +536,7 @@ if present.
 						int32_t d = (int32_t)bswap_32(*(uint32_t*)(data+bi[k]+m*bi[NS]));
 						((double*)mxGetData(R[k]))[m] = (d>2147483620) ? NaN : d;
 					}
-					else for (size_t m = 0; m < M; m++) {
+					else for (typeof(M) m = 0; m < M; m++) {
 						int32_t d = *(int32_t*)(data+bi[k]+m*bi[NS]);
 						((double*)mxGetData(R[k]))[m] = (d>2147483620) ? NaN : d;
 					}
@@ -562,17 +552,17 @@ if present.
 					break;
 				case 0xff:
 					R[k] = mxCreateDoubleMatrix(M, 1, mxREAL);
-					if (flagSWAP) for (size_t m = 0; m < M; m++) {
+					if (flagSWAP) for (typeof(M) m = 0; m < M; m++) {
 						((uint64_t*)mxGetData(R[k]))[m] = bswap_64(*(uint64_t*)(data+bi[k]+m*bi[NS]));
 					}
-					else for (size_t m = 0; m < M; m++) {
+					else for (typeof(M) m = 0; m < M; m++) {
 						((uint64_t*)mxGetData(R[k]))[m] = *(uint64_t*)(data+bi[k]+m*bi[NS]);
 					}
 					break;
 				default:
 					R[k] =	mxCreateCellMatrix(M, 1);
 					size_t sz = typlist[k];
-					for (size_t m = 0; m < M; m++) {
+					for (typeof(M) m = 0; m < M; m++) {
 						memcpy(f, data+bi[k]+m*bi[NS], sz);
 						f[sz] = 0;
 						mxSetCell(R[k], m, mxCreateString(f));
@@ -632,11 +622,11 @@ if present.
 			int status = 0;
 			while (line) {
 
-				if (!strncmpi(line,"@relation",9)) {
+				if (!strncasecmp(line,"@relation",9)) {
 					status = 1;
 				}
 
-				else if (status == 1 && !strncmpi(line,"@attribute",10)) {
+				else if (status == 1 && !strncasecmp(line,"@attribute",10)) {
 					if (ns<=NS) {
 						ns = max(16, ns*2);
 						ListOfVarNames = (const char**)realloc(ListOfVarNames,ns*sizeof(char*));
@@ -653,22 +643,22 @@ if present.
 					p2 = line+k;
 
 					ListOfVarNames[NS] = p1;
-					if      (!strncmpi(p2,"numeric",7)) {
+					if      (!strncasecmp(p2,"numeric",7)) {
 						vartyp[NS] = 1;
 					}
-					else if (!strncmpi(p2,"integer",7)) {
+					else if (!strncasecmp(p2,"integer",7)) {
 						vartyp[NS] = 2;
 					}
-					else if (!strncmpi(p2,"real",4)) {
+					else if (!strncasecmp(p2,"real",4)) {
 						vartyp[NS] = 3;
 					}
-					else if (!strncmpi(p2,"string",6)) {
+					else if (!strncasecmp(p2,"string",6)) {
 						vartyp[NS] = 4;
 					}
-					else if (!strncmpi(p2,"{",1)) {
+					else if (!strncasecmp(p2,"{",1)) {
 						vartyp[NS] = 5;
 					}
-					else if (!strncmpi(p2,"date",4)) {
+					else if (!strncasecmp(p2,"date",4)) {
 						vartyp[NS] = 6;
 						datestr = (char**)realloc(datestr,(NS+1)*sizeof(char*));
 						p2+=4;
@@ -680,7 +670,7 @@ if present.
 							p2[1]=0;
 						}
 					}
-					else if (!strncmpi(p2,"relational",10)) {
+					else if (!strncasecmp(p2,"relational",10)) {
 						vartyp[NS] = 7;
 					}
 					else vartyp[NS] = 99;
@@ -688,7 +678,7 @@ if present.
 					NS++;
 				}
 
-				else if (status == 1 && !strncmpi(line,"@data",5)) {
+				else if (status == 1 && !strncasecmp(line,"@data",5)) {
 					status = 2;
 					char *p = line;
 					while (*p) p++;  // goto end of current line
@@ -738,7 +728,7 @@ if present.
 						else if (vartyp[ns] == 6) {
 							size_t kk[6],n=0, N=strlen(datestr[ns]);
 							char T0[6][5];
-							char ix = 0;
+							int ix = 0;
 							struct tm t;
 
 							for (n=0; n < N; n++) {
@@ -947,7 +937,7 @@ if present.
 		H2 = (char*) realloc(H2,HeadLen2);
 		memset(H2,0,HeadLen2);
 
-		mwIndex M = 0;
+		size_t M = 0;
 		mxArray **F = (mxArray**) mxMalloc(NS*sizeof(mxArray*));
 		char **Fstr = (char**) malloc(NS*sizeof(char*));
 		size_t *MAXLEN = (size_t*) malloc(NS*sizeof(size_t*));
@@ -968,7 +958,7 @@ if present.
 			}
 			else if (mxIsCell(F[k])) {
 				size_t maxlen = 0;
-				for (mwIndex m = 0; m<M; m++) {
+				for (size_t m = 0; m<M; m++) {
 					mxArray *f = mxGetCell(F[k],m);
 					if (mxIsChar(f) || mxIsEmpty(f)) {
 						size_t len = mxGetNumberOfElements(f);
@@ -992,7 +982,7 @@ if present.
 		count += fwrite(H2, 1, HeadLen2, fid);
 		/* write OBS header line */
 		count += fwrite(LO, 1, strlen(LO), fid);
-		for (mwIndex m = 0; m < M; m++) {
+		for (size_t m = 0; m < M; m++) {
 			for (uint16_t k = 0; k < NS; k++) {
 
 				if (*(int16_t*)(H2+k*sz2) == b_endian_u16(1)) {
@@ -1106,7 +1096,7 @@ uint64_t d2xpt(double x) {
 	mexPrintf("d2xpt(%f)\n",x);
 #endif
 	// see http://old.nabble.com/Re%3A-IBM-integer-and-double-formats-p20428979.html
-	m = *(uint64_t*) &x;
+	memcpy(&m, &x, 8);
 	*(((char*)&m) + 6) &= 0x0f; //
 	if (e) *(((char*)&m) + 6) |= 0x10; // reconstruct implicit leading '1' for normalized numbers
 	m <<= (3-(-e & 3));
